% Gavin McNicol - EPS 209 Matlab for Earth Scientists

% This function:
 
% (1) Gets names of cow images in directory
% (2) Allows user creation of color polygon ( I used green grass patch )
%     * NOTE: Specify 'grassy' image by changing Line 27 - listing(n) to 
%             suitable image number. Image numbering explained in README.
% (3) Loop over all image files
% (4) Create vector of days based upon number of images per day
% (5) Plot mean Mahalanobis distance against time

%% (1) Get names of cow images in directory
clear all; close all

% Get directory listing
listing = dir('*.jpg');
% Get number of files matching the query
nFiles = numel(listing);
% Print all the file names using a loop
for n=1:nFiles
fprintf([listing(n).name '\n']);
end 
 

%% (2) Create color polygon
% Load ROI file
ROI = imread('ROIwhole.png');
 
% Get image name
imName = listing(n).name;
  
% Read image
I = imread(imName);
  
% Convert image to black and white
IBW = rgb2gray(I);
IBW = imadjust(IBW);

% Filter the image by the ROI
I_red = immultiply(ROI,I(:,:,1));
I_green = immultiply(ROI,I(:,:,2));
I_blue = immultiply(ROI,I(:,:,3));
Iroi = cat(3, I_red, I_green, I_blue); 
IBWroi = rgb2gray(Iroi);
  
% Get size
[nRows nCols nLyrs] = size(Iroi);
 nPix = nRows * nCols;
 
% set filter size
filtSize = 5;
   
% smooth image
fprintf('Smoothing image ...\n');
myFilt = ones(filtSize) ./ filtSize^2;
smoothImg = imfilter(Iroi, myFilt);
   
% print some instructions with the function fprintf()
fprintf('Use mouse to delineate region for color sample\n');
fprintf('Click to place a vertex\n');
fprintf('Right-click and select Create Mask to close polygon\n');
% open a new figure for the region selection and call roipoly()
figure;
title('Select region');
roiMask = roipoly(Iroi);
% display the mask
figure;
imshow(roiMask);
title('Mask');
% use the mask to extract the color region
fprintf('Masking region ...\n');
red = immultiply(roiMask, smoothImg(:,:,1));
green = immultiply(roiMask, smoothImg(:,:,2));
blue = immultiply(roiMask, smoothImg(:,:,3));
roiImg = cat(3, red, green, blue);
% display the color region
figure;
imshow(roiImg);
title('Color sample');
   
% reshape image into vector of size nPix by 3 
imgPix = double(reshape(smoothImg, nPix, 3));
% EXTRACT NONZERO ENTRIES
imgPix = nonzeros(imgPix);

% reshape roi region into vector of size nPix by 3
roiPix = double(reshape(roiImg, nPix, 3));
% find the nonzero entries in the mask
roiIdx = find(roiMask);
% keep only the RGB pixels corresponding to the mask
roiPix = roiPix(roiIdx, 1:3);
% get statistics
fprintf('Computing region statistics ...\n');
% mean color (could be used for drawing the forest in a "truer" color)
myMean = mean(roiPix)
% standard deviation (use the max value for R, G, or B)
myStd = max(std(roiPix))
 
%% (3) Loop over all image files
for n=1:nFiles
    
  % Get image name
  imName = listing(n).name;
  
  % Read image
  I = imread(imName);

  % Convert image to black and white
  IBW = rgb2gray(I);

  % Filter the image by the ROI
  I_red = immultiply(ROI,I(:,:,1));
  I_green = immultiply(ROI,I(:,:,2));
  I_blue = immultiply(ROI,I(:,:,3));
  Iroi = cat(3, I_red, I_green, I_blue); 
  IBWroi = rgb2gray(Iroi);
  
  % Get size
  [nRows nCols nLyrs] = size(Iroi);
   nPix = nRows * nCols;
 
   % set filter size
   filtSize = 5;
   
   % smooth image
   fprintf('Smoothing image ...\n');
   myFilt = ones(filtSize) ./ filtSize^2;
   smoothImg = imfilter(Iroi, myFilt);
   
   % reshape image into vector of size nPix by 3
   imgPix = double(reshape(smoothImg, nPix, 3));
            
   % compute distance vector
   mahDist = mahal(imgPix, roiPix);
   % reshape distance and store in matrix
   myDist = reshape(mahDist, nRows, nCols);

   % Mean
   y(n) = mean2(myDist);

   % % display the distance
   % figure;
   % imagesc(myDist);
   % title('Mahalanobis distance from region color');
   % colorbar;
   % impixelinfo

   % % define how many standard deviations are allowed
   % kThresh = 1;
   % % myDist contains the distances of each pixel to mean color
   % % if minimum is more than k*std label it 0, else label 1
   % fprintf('Thresholding pixels with distance from region color ...\n');
   % mySegm = false(nRows, nCols);
   % mySegm(myDist <= kThresh * myStd) = true;
   % % get the red, green and blue parts of the image
   % Ir = Iroi(:,:,1);
   % Ig = Iroi(:,:,2);
   % Ib = Iroi(:,:,3);
   % % label the segmented pixels in red
   % Ir(mySegm==1) = 255;
   % Ib(mySegm==1) = 0;
   % Ig(mySegm==1) = 0;
   % % put the red, green, and blue parts back together
   % Iroi2 = cat(3,Ir,Ig,Ib);
   % % display the labeled image
   % figure; imshow(Iroi2);
   % title('Classified forest on original image'); 
   % 
   %   if y(n) > 6.5
   %       fprintf([listing(n).name '\n']);
   %   end
end

%% (4) Create vector of days based upon number of images per day
for i = 1:nFiles
    x(i) = i/24;
end

%% (5) Plot Mahalanobis mean mahalanobis distance against Time 

figure, plot(x,y); xlabel('Time (Days)'); ylabel('Mean Mahalanobis Distance');



